BrokerCtA = LibStub("AceAddon-3.0"):NewAddon("BrokerCtA", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("BrokerCtA", true)

local LSM = LibStub("LibSharedMedia-3.0")
LSM:Register("sound", "War Drums", [[Sound\Event Sounds\Event_wardrum_ogre.wav]])

local UPDATEPERIOD, elapsed = 5, 0

BrokerCtA.display = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Broker_CallToArms", {type = 'data source', label=L["CtA"], text = L["..."]})
BrokerCtA.shortageStatusCounts = {
	Tank = { Rare = 0, Uncommon = 0, Plentiful = 0 },
	Healer = { Rare = 0, Uncommon = 0, Plentiful = 0 },
	DPS = { Rare = 0, Uncommon = 0, Plentiful = 0 },
}

BrokerCtA.updateframe = CreateFrame("frame")

local shortageColors = {
	Rare = "|cFFFF0000",
	Uncommon = "|cFFFFFF00",
	Plentiful = "|cFFCCCCCC",
}

local roleIcons = {
	Tank = '|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:0:19:20:39|t',
	Healer = '|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:0:19|t',
	DPS = '|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:20:39|t',
}

local shortageStatus = {}

local availableRoles = {}

function BrokerCtA:OnInitialize()

	availableRoles["Tank"], availableRoles["Healer"], availableRoles["DPS"] = UnitGetAvailableRoles("player")

	self.options = {
		name = L["Broker - Call to Arms: Options"],
		handler = BrokerCtA,
		type = 'group',
		args = {
			settings = {
				name = 'settings',
				type = 'group',
				args = {
					addon_enabled = {
						type = 'toggle',
						name = L['Enabled'],
						desc = L['Check to enable monitoring and notification of Call to Arms.'],
						set = function(info,val)
							if val then
								BrokerCtA:Enable()
							else
								BrokerCtA:Disable()
							end
							BrokerCtA.db.profile.enabled = val
						end,
						get = function(info) return BrokerCtA:IsEnabled() end,
						order = 0,
					},
					spacer1 = {
						type = 'description', 
						name = "\n",
						order = 1,
					},
					general = {
						type = 'group',
						name = L['General'],
						order = 2,
						args = {
							ldbDisplay = {
								type = 'group',
								name = L['LDB Display Options'],
								inline = true,
								order = 1,
								args = {
									shortLabel = {
										type = 'toggle',
										name = L['Short Label'],
										desc = L['Display "CtA" on the LDB text instead of "Call To Arms"'],
										order = 1,
										set = function(info, val)
											BrokerCtA.db.profile.shortLabel = val
											BrokerCtA:RefreshLDB()
										end,
										get = function()
											return BrokerCtA.db.profile.shortLabel
										end,
									},
									shortText = {
										type = 'toggle',
										name = L['Short Text'],
										desc = L['Display "T", "H" and "D" on the LDB text instead of "Tank", "Healer" and "DPS"'],
										order = 2,
										set = function(info, val)
											BrokerCtA.db.profile.shortText = val
											BrokerCtA:RefreshLDB()
										end,
										get = function()
											return BrokerCtA.db.profile.shortText
										end,
									},
								},
							},
							spacer1 = {
								type = 'description',
								name = "\n",
								order = 2,
							},
							priority = {
								type = 'select',
								name = L['Auto-Queue'],
								desc = L['Automatically enter the queue for the following role when CtA begins for it.'],
								order = 3,
								width = 'full',
								values = {
									Tank = L["Tank"],
									Healer = L["Healer"],
									DPS = L["DPS"],
								},
								set = function (_, val)
									BrokerCtA.autoQueue = val
								end,
								get = function ()
									return BrokerCtA.autoQueue
								end,
							},
						},
					},
					queues = {
						type = 'group',
						name = L['Dungeon Groups'],
						desc = L['Select which dungeon queues you wish to watch for Calls to Arms.'],
						order = 3,
						args = {},
					},
					Tank = {
						type = 'group',
						name = '|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:0:19:20:39|t'..L['Tank'],
						order = 4,
						args = {
							monitor = {
								type = 'select',
								name = L['Monitor Tank Role'],
								desc = L['Under what circumstances should this role be monitored?'],
								values = {
									[1] = L['Never'],
									[2] = L['Always'],
									[3] = L['If selected in LFD'],
									[4] = L['If I can fill the role'],
								},
								style = 'dropdown',
								order = 1,
								set = function(info, val)
									BrokerCtA.db.profile.Tank.monitor = val
								end,
								get = function()
									return BrokerCtA.db.profile.Tank.monitor
								end,
							},
							alert = {
								type = 'select',
								name = '|cFFFFCC00'..L['Alert When'],
								values = {
									[1] = L['Rare'],
									[2] = L['Uncommon'],
									[3] = L['Either'],
								},
								order = 2,
								set = function(info, val)
									BrokerCtA.db.profile.Tank.alert = val
								end,
								get = function()
									return BrokerCtA.db.profile.Tank.alert
								end,
							},
							text = {
								type = 'toggle',
								name = '|cFFFFCC00'..L['Text Alert'],
								desc = L['Display message on screen when CtA becomes available for this role.'],
								descStyle = 'inline',
								order = 4,
								set = function(info, val)
									BrokerCtA.db.profile.Tank.textNotify = val
								end,
								get = function()
									return BrokerCtA.db.profile.Tank.textNotify
								end,
							},
							chat = {
								type = 'toggle',
								name = '|cFFFFCC00'..L['Chat Alert'],
								desc = L['Display message in chat window when CtA becomes available for this role.'],
								descStyle = 'inline',
								order = 5,
								set = function(info, val)
									BrokerCtA.db.profile.Tank.chatNotify = val
								end,
								get = function()
									return BrokerCtA.db.profile.Tank.chatNotify
								end,
							},
							soundFile = {
								type = 'select',
								dialogControl = 'LSM30_Sound',
								name = L['Audible Alert'],
								desc = L['Select a sound file to be played when CtA becomes available for this role.'],
								values = LSM:HashTable("sound"),
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									BrokerCtA.db.profile.Tank.soundFile = val
								end,
								get = function()
									return BrokerCtA.db.profile.Tank.soundFile
								end,
							},
						},
					},
					Healer = {
						type = 'group',
						name = '|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:0:19|t'..L['Healer'],
						order = 4,
						args = {
							monitor = {
								type = 'select',
								name = L['Monitor Healer Role'],
								desc = L['Under what circumstances should this role be monitored?'],
								values = {
									[1] = L['Never'],
									[2] = L['Always'],
									[3] = L['If selected in LFD'],
									[4] = L['If I can fill the role'],
								},
								style = 'dropdown',
								order = 1,
								set = function(info, val)
									BrokerCtA.db.profile.Healer.monitor = val
								end,
								get = function()
									return BrokerCtA.db.profile.Healer.monitor
								end,
							},
							alert = {
								type = 'select',
								name = '|cFFFFCC00'..L['Alert When'],
								values = {
									[1] = L['Rare'],
									[2] = L['Uncommon'],
									[3] = L['Either'],
								},
								order = 2,
								set = function(info, val)
									BrokerCtA.db.profile.Healer.alert = val
								end,
								get = function()
									return BrokerCtA.db.profile.Healer.alert
								end,
							},
							text = {
								type = 'toggle',
								name = '|cFFFFCC00'..L['Text Alert'],
								desc = L['Display message on screen when CtA becomes available for this role.'],
								descStyle = 'inline',
								order = 4,
								set = function(info, val)
									BrokerCtA.db.profile.Healer.textNotify = val
								end,
								get = function()
									return BrokerCtA.db.profile.Healer.textNotify
								end,
							},
							chat = {
								type = 'toggle',
								name = '|cFFFFCC00'..L['Chat Alert'],
								desc = L['Display message in chat window when CtA becomes available for this role.'],
								descStyle = 'inline',
								order = 5,
								set = function(info, val)
									BrokerCtA.db.profile.Healer.chatNotify = val
								end,
								get = function()
									return BrokerCtA.db.profile.Healer.chatNotify
								end,
							},
							soundFile = {
								type = 'select',
								dialogControl = 'LSM30_Sound',
								name = L['Audible Alert'],
								desc = L['Select a sound file to be played when CtA becomes available for this role.'],
								values = LSM:HashTable("sound"),
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									BrokerCtA.db.profile.Healer.soundFile = val
								end,
								get = function()
									return BrokerCtA.db.profile.Healer.soundFile
								end,
							},
						},
					},
					dps = {
						type = 'group',
						name = '|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:20:39|t'..L['DPS'],
						order = 4,
						args = {
							monitor = {
								type = 'select',
								name = L['Monitor DPS Role'],
								desc = L['Under what circumstances should this role be monitored?'],
								values = {
									[1] = L['Never'],
									[2] = L['Always'],
									[3] = L['If selected in LFD'],
									[4] = L['If I can fill the role'],
								},
								style = 'dropdown',
								order = 1,
								set = function(info, val)
									BrokerCtA.db.profile.DPS.monitor = val
								end,
								get = function()
									return BrokerCtA.db.profile.DPS.monitor
								end,
							},
							alert = {
								type = 'select',
								name = '|cFFFFCC00'..L['Alert When'],
								values = {
									[1] = L['Rare'],
									[2] = L['Uncommon'],
									[3] = L['Either'],
								},
								order = 2,
								set = function(info, val)
									BrokerCtA.db.profile.DPS.alert = val
								end,
								get = function()
									return BrokerCtA.db.profile.DPS.alert
								end,
							},
							text = {
								type = 'toggle',
								name = '|cFFFFCC00'..L['Text Alert'],
								desc = L['Display message on screen when CtA becomes available for this role.'],
								descStyle = 'inline',
								order = 4,
								set = function(info, val)
									BrokerCtA.db.profile.DPS.textNotify = val
								end,
								get = function()
									return BrokerCtA.db.profile.DPS.textNotify
								end,
							},
							chat = {
								type = 'toggle',
								name = '|cFFFFCC00'..L['Chat Alert'],
								desc = L['Display message in chat window when CtA becomes available for this role.'],
								descStyle = 'inline',
								order = 5,
								set = function(info, val)
									BrokerCtA.db.profile.DPS.chatNotify = val
								end,
								get = function()
									return BrokerCtA.db.profile.DPS.chatNotify
								end,
							},
							soundFile = {
								type = 'select',
								dialogControl = 'LSM30_Sound',
								name = L['Audible Alert'],
								desc = L['Select a sound file to be played when CtA becomes available for this role.'],
								values = AceGUIWidgetLSMlists.sound,
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									BrokerCtA.db.profile.DPS.soundFile = val
								end,
								get = function()
									return BrokerCtA.db.profile.DPS.soundFile
								end,
							},
						},
					},
				},
			},
		},
	}
	
	self.defaults = {
		profile = {
			enabled = true,
			shortLabel = false,
			shortText = false,
			queues = {
				['auto'] = true,
			},
			Tank = {
				monitor = 3,
				alert = 1,
				textNotify = true,
				chatNotify = true,
				soundFile = 'War Drums',
			},
			Healer = {
				monitor = 3,
				alert = 1,
				textNotify = true,
				chatNotify = true,
				soundFile = 'War Drums',
			},
			DPS = {
				monitor = 3,
				alert = 1,
				textNotify = true,
				chatNotify = true,
 				soundFile = 'War Drums',
			},
		}
	}
	
	self.db = LibStub("AceDB-3.0"):New("BrokerCTAOptions",self.defaults,"Default")
	
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("BrokerCtA", self.options)
	
	self.settingsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BrokerCtA", L["Broker Call To Arms"], nil, "settings")  
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BrokerCtA", L["Profiles"], L["Broker Call To Arms"], "profiles")
	
	if not self.db.profile.enabled then self:Disable() end
	
	self.updateframe:SetScript("OnUpdate",BrokerCtA.UpdateFrameUpdate)
	
end

function BrokerCtA:OnLFGInitialize()
	local t = {
		['auto'] = {
			type = 'toggle',
			name = '|cFFFFCC00'..L['Auto-recommend'],
			set = function(info, val)
				BrokerCtA.db.profile.queues = {} 
				BrokerCtA.db.profile.queues['auto'] = val
				BrokerCtA.db.profile.queues[tostring(GetRandomDungeonBestChoice())] = true
			end,
			get = function()
				return BrokerCtA.db.profile.queues.auto
			end,
			order = 0,
		}
	}
	
	for i=1, GetNumRandomDungeons() do
		local id, name = GetLFGRandomDungeonInfo(i)
		local _, _, _, _, itemCount = GetLFGRoleShortageRewards(id, 1)
		if itemCount > 0 then
			shortageStatus[tostring(id)] = {
				Name = name,
				Status = {Tank="", Healer="", DPS=""},
			}
			t[tostring(id)] = {
				type = 'toggle',
				name = strsub(name, 8),
				disabled = function() return BrokerCtA.db.profile.queues.auto end, 
				set = function(info, val)
					BrokerCtA.db.profile.queues[id] = val
				end,
				get = function()
					return (BrokerCtA.db.profile.queues[id] and not BrokerCtA.db.profile.queues.auto) or (BrokerCtA.db.profile.queues.auto and GetRandomDungeonBestChoice() == id)
				end,
				order = 10000 - id,
			}
		end
	end
	self.options.args.settings.args.queues.args = t
	self.LFGInitialized = true 
	RequestLFDPlayerLockInfo()
end

function BrokerCtA.UpdateFrameUpdate(self, elap)

	if not BrokerCtA.LFGInitialized then
		elapsed = elapsed + elap
		if elapsed < .5 then return end
		elapsed = 0
		if GetNumRandomDungeons() > 0 then 
			BrokerCtA:OnLFGInitialize()
		end
		return
	end

	if not BrokerCtA:IsEnabled() then return end

	elapsed = elapsed + elap
	if elapsed < UPDATEPERIOD then return end
	elapsed = 0

	RequestLFDPlayerLockInfo()

end

function BrokerCtA:OnEnable()
	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO")
	if self.LFGInitialized then RequestLFDPlayerLockInfo() end
end

function BrokerCtA:OnDisable()
	BrokerCtA.display.text = "|cFF808080"..L["Disabled"]
end

local function IsMonitored(role)
	if BrokerCtA.db.profile[role].monitor == 1 then --NEVER
		return false
	elseif BrokerCtA.db.profile[role].monitor == 2 then --ALWAYS
		return true
	elseif BrokerCtA.db.profile[role].monitor == 3 then --SELECTED
		local LFGRoles = {}
		_, LFGRoles.Tank, LFGRoles.Healer, LFGRoles.DPS = GetLFGRoles()
		return LFGRoles[role]
	elseif BrokerCtA.db.profile[role].monitor == 4 then --AUTO
		return availableRoles(role)
	end
end

function BrokerCtA:LFG_UPDATE_RANDOM_INFO ()

	if not self.LFGInitialized then return end

	local eligible, forTank, forHealer, forDPS, itemCount
	local newStatuses = {}

	for i=1, GetNumRandomDungeons() do
		local dungeonGroup, dungeonName = GetLFGRandomDungeonInfo(i);
		dungeonGroup = tostring(dungeonGroup)

		eligible, forTank, forHealer, forDPS, itemCount = GetLFGRoleShortageRewards(tonumber(dungeonGroup), 1);
		if itemCount > 0 then
			newStatuses[dungeonGroup] = {
				Name = dungeonName,
				Status = {Tank = "Plentiful", Healer = "Plentiful", DPS = "Plentiful"},
			}
			
			if forTank then newStatuses[dungeonGroup].Status.Tank = "Rare" end
			if forHealer then newStatuses[dungeonGroup].Status.Healer = "Rare" end
			if forDPS then newStatuses[dungeonGroup].Status.DPS = "Rare" end
			
			eligible, forTank, forHealer, forDPS, itemCount = GetLFGRoleShortageRewards(tonumber(dungeonGroup), 2);
			if forTank then newStatuses[dungeonGroup].Status.Tank = "Uncommon" end
			if forHealer then newStatuses[dungeonGroup].Status.Healer = "Uncommon" end
			if forDPS then newStatuses[dungeonGroup].Status.DPS = "Uncommon" end

			newStatuses[dungeonGroup].Eligible = eligible
			newStatuses[dungeonGroup].Monitored = (self.db.profile.queues.auto and tonumber(dungeonGroup) == GetRandomDungeonBestChoice()) or (self.db.profile.queues[dungeonGroup] and not self.db.profile.queues.auto)
		end
	end
	
	self.shortageStatusCounts = {
		Tank = { Rare = 0, Uncommon = 0, Plentiful = 0 },
		Healer = { Rare = 0, Uncommon = 0, Plentiful = 0 },
		DPS = { Rare = 0, Uncommon = 0, Plentiful = 0 },
	}

	for dungeonGroup, dungeonData in pairs(newStatuses) do
		if (self.db.profile.queues.auto and tonumber(dungeonGroup) == GetRandomDungeonBestChoice()) or (self.db.profile.queues[dungeonGroup] and not self.db.profile.queues.auto) then
			self.shortageStatusCounts.Tank[dungeonData.Status.Tank] = self.shortageStatusCounts.Tank[dungeonData.Status.Tank] + 1
			self.shortageStatusCounts.Healer[dungeonData.Status.Healer] = self.shortageStatusCounts.Healer[dungeonData.Status.Healer] + 1
			self.shortageStatusCounts.DPS[dungeonData.Status.DPS] = self.shortageStatusCounts.DPS[dungeonData.Status.DPS] + 1
			if shortageStatus[dungeonGroup].Status.Tank ~= dungeonData.Status.Tank then self:ProcessShortageStatusChange(dungeonGroup, "Tank", dungeonData.Status.Tank) end
			if shortageStatus[dungeonGroup].Status.Healer ~= dungeonData.Status.Healer then self:ProcessShortageStatusChange(dungeonGroup, "Healer", dungeonData.Status.Healer) end
			if shortageStatus[dungeonGroup].Status.DPS ~= dungeonData.Status.DPS then self:ProcessShortageStatusChange(dungeonGroup, "DPS", dungeonData.Status.DPS) end
		end
		shortageStatus[dungeonGroup] = dungeonData
	end
	
end

function BrokerCtA:ProcessShortageStatusChange(dungeonGroup, role, newStatus)

	BrokerCtA:RefreshLDB()

	if not IsMonitored(role) then return end
	if newStatus == "Rare" and bit.band(BrokerCtA.db.profile[role].alert, 1) == 0 then return end
	if newStatus == "Uncommon" and bit.band(BrokerCtA.db.profile[role].alert, 2) == 0 then return end
	if newStatus == "Plentiful" then return end
	
	if BrokerCtA.db.profile[role].textNotify then
		RaidNotice_AddMessage( RaidBossEmoteFrame, L["Call To Arms: "]..roleIcons[role]..shortageColors[newStatus]..L[role].."|r - "..shortageStatus[dungeonGroup].Name, ChatTypeInfo["RAID_BOSS_EMOTE"] )
	end
	
	if BrokerCtA.db.profile[role].chatNotify then
		ChatFrame1:AddMessage( L["Call To Arms: "]..roleIcons[role]..shortageColors[newStatus]..L[role].."|r - "..shortageStatus[dungeonGroup].Name, 1, .8, 0, 1 )
	end
	
	if BrokerCtA.db.profile[role].soundFile ~= "" and BrokerCtA.db.profile[role].soundFile ~= "None" then
		PlaySoundFile(LSM:Fetch("sound",BrokerCtA.db.profile[role].soundFile),"Master")
	end
	
	if BrokerCtA.autoQueue == role then
		BrokerCtA:EnterQueue(dungeonGroup, role)
	end
end

function BrokerCtA:EnterQueue(dungeonGroup, role)
	local LFGRoles = {}
	LFGRoles.Leader, LFGRoles.Tank, LFGRoles.Healer, LFGRoles.DPS = GetLFGRoles()
	if role then
		LFGRoles.Tank, LFGRoles.Healer, LFGRoles.DPS = false, false, false
		LFGRoles[role] = true
		BrokerCtA.autoQueue = nil
	end
	SetLFGRoles(LFGRoles.Leader, LFGRoles.Tank, LFGRoles.Healer, LFGRoles.DPS)
	if dungeonGroup then 
		SetLFGDungeon(dungeonGroup)
	elseif LFDQueueFrame.type then
		SetLFGDungeon(LFDQueueFrame.type)
	else
		SetLFGDungeon(GetRandomDungeonBestChoice())
	end
	JoinLFG()
end

function BrokerCtA:RefreshLDB()
	self.display.label = self.db.profile.shortLabel and L['CtA'] or L['Call To Arms'] 
	self.display.icon = "Interface\\ICONS\\Achievement_Arena_2v2_3"

	local text = ""
	if bit.band(1, self.db.profile.Tank.alert) and self.shortageStatusCounts.Tank.Rare > 0 then
		text = text..shortageColors.Rare..(self.db.profile.shortText and L['T'] or L['Tank'])..(self.shortageStatusCounts.Tank.Rare > 1 and " ("..self.shortageStatusCounts.Tank.Rare..") " or " ")
	elseif bit.band(2, self.db.profile.Tank.alert) and self.shortageStatusCounts.Tank.Uncommon > 0 then
		text = text..shortageColors.Uncommon..(self.db.profile.shortText and L['T'] or L['Tank'])..(self.shortageStatusCounts.Tank.Uncommon > 1 and " ("..self.shortageStatusCounts.Tank.Uncommon..") " or " ")
	end
	if bit.band(1, self.db.profile.Healer.alert) and self.shortageStatusCounts.Healer.Rare > 0 then
		text = text..shortageColors.Rare..(self.db.profile.shortText and L['H'] or L['Healer'])..(self.shortageStatusCounts.Healer.Rare > 1 and " ("..self.shortageStatusCounts.Healer.Rare..") " or " ")
	elseif bit.band(2, self.db.profile.Healer.alert) and self.shortageStatusCounts.Healer.Uncommon > 0 then
		text = text..shortageColors.Uncommon..(self.db.profile.shortText and L['H'] or L['Healer'])..(self.shortageStatusCounts.Healer.Uncommon > 1 and " ("..self.shortageStatusCounts.Healer.Uncommon..") " or " ")
	end
	if bit.band(1, self.db.profile.DPS.alert) and self.shortageStatusCounts.DPS.Rare > 0 then
		text = text..shortageColors.Rare..(self.db.profile.shortText and L['D'] or L['DPS'])..(self.shortageStatusCounts.DPS.Rare > 1 and " ("..self.shortageStatusCounts.DPS.Rare..") " or " ")
	elseif bit.band(2, self.db.profile.DPS.alert) and self.shortageStatusCounts.DPS.Uncommon > 0 then
		text = text..shortageColors.Uncommon..(self.db.profile.shortText and L['D'] or L['DPS'])..(self.shortageStatusCounts.DPS.Uncommon > 1 and " ("..self.shortageStatusCounts.DPS.Uncommon..") " or " ")
	end
	self.display.text = (text ~= "") and text or "|cFF808080"..L['N/A']
end

function BrokerCtA.display:OnEnter()
	local tooltip = LibStub('LibQTip-1.0'):Acquire("BrokerCtATooltip", 5, "CENTER", "LEFT", "CENTER", "CENTER", "CENTER")
	local checkmark = "|TInterface\\BUTTONS\\UI-CheckBox-Check:20:20|t"
	self.tooltip = tooltip 
	tooltip:AddHeader('',L['Dungeon Group'], L['Tank'], L['Healer'], L['DPS'])
	for dungeonGroup, dungeonData in pairs(shortageStatus) do
		tooltip:AddLine((dungeonData.Monitored and checkmark or ""), (dungeonData.Eligible and '|cFFFFCC00' or '|cFFCCCCCC')..dungeonData.Name, shortageColors[dungeonData.Status.Tank]..L[dungeonData.Status.Tank], shortageColors[dungeonData.Status.Healer]..L[dungeonData.Status.Healer], shortageColors[dungeonData.Status.DPS]..L[dungeonData.Status.DPS].."|r ")
	end
	tooltip:SmartAnchorTo(self)
	tooltip:Show()
end

function BrokerCtA.display:OnLeave()
   -- Release the tooltip
   LibStub('LibQTip-1.0'):Release(self.tooltip)
   self.tooltip = nil
end

function BrokerCtA.display:OnClick(button)
	if IsMouseButtonDown(button) then return end
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(BrokerCtA.settingsFrame)
	end
	if button == "LeftButton" then
		if IsControlKeyDown() then
			BrokerCtA:EnterQueue()
		else
			ToggleLFDParentFrame()
		end
	end
end